// glSDL-Benchmark (based on testsprite.c of SDL).
// Roberto Prieto - contact@megastormsystems.com
// MegaStorm Systems (c) 2013 - http://www.megastormsystems.com
// v2.0 -> added external test for SDL 2.0 renderers (direct3d, opengl and software)
// v1.3 -> removed external tests(they dont work anymore) and compiled using VS2008
// v1.2 -> Added windib rendering.
// v1.1 -> Added software test and two external: ddraw and d3d.
// v1.0 -> Minor changes and html output
// v0.9 -> First version (beta test)

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <windows.h>
#include <winerror.h>
#include "sdl/SDL.h"
#include "sdl/SDL_opengl.h"

#define GLSDL_VERSION "glSDL Benchmark v2.0"
#define DEBUG_FLIP     0
//#define DEBUG_BACKEND
#define NUM_SPRITES	1000
#define NUM_SPRITES2 100
#define MAX_SPEED 	   1
#define STEP_TIME     10

SDL_Surface *sprite,*bg;
int numsprites;
SDL_Rect *sprite_rects;
SDL_Rect *positions;
SDL_Rect *velocities;
int sprites_visible;
Uint16 sprite_w, sprite_h;
const SDL_VideoInfo *vi;
char buf[20];
FILE *fLog;

#define NUM_TEST 12
struct glTest
{
	Uint16 bMode;  // 0: DirectX 1: glSDL 2: WinDib
	Uint32 bVideo; // SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF
	Uint16 iLoad;  // 1000 sprites or 100 sprites
	double fps;    // Store frames per second
} glTest[12];

// Prototypes
int LoadSprite(SDL_Surface *screen, char *file);
void MoveSprites(SDL_Surface *screen, Uint32 background);
bool GetProcessorInfo(void);
bool GetMemoryInfo(void);
bool GetVideocardInfo(void);


// Main body
int main(int argc, char *argv[])
{
	SDL_Surface *screen;
	Uint8 *mem;
	int width, height,iLast;
	Uint8  video_bpp;
	Uint32 videoflags;
	int    i, done,iTest;
	Uint32 then, now, frames,iTime;

	// Mensaje de aviso
	MessageBox(NULL,"This test takes about 180 seconds.\nPlease, wait...",GLSDL_VERSION,MB_OK);
	fLog = fopen("glSDL-Bench.htm","wt");

	// Inicializa sistema de test	
	//glTest[0].bMode=1; glTest[0].bVideo=SDL_HWSURFACE|SDL_DOUBLEBUF; glTest[0].iLoad=NUM_SPRITES;
	glTest[0].bMode=0; glTest[0].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF; glTest[0].iLoad=NUM_SPRITES2;
	glTest[1].bMode=0; glTest[1].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF; glTest[1].iLoad=NUM_SPRITES;
	glTest[2].bMode=0; glTest[2].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN; glTest[2].iLoad=NUM_SPRITES2;
	glTest[3].bMode=0; glTest[3].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN; glTest[3].iLoad=NUM_SPRITES;
	glTest[4].bMode=1; glTest[4].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF; glTest[4].iLoad=NUM_SPRITES2;
	glTest[5].bMode=1; glTest[5].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF; glTest[5].iLoad=NUM_SPRITES;
	glTest[6].bMode=1; glTest[6].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN; glTest[6].iLoad=NUM_SPRITES2;
	glTest[7].bMode=1; glTest[7].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN; glTest[7].iLoad=NUM_SPRITES;
	glTest[8].bMode=0; glTest[8].bVideo=SDL_SWSURFACE|SDL_FULLSCREEN; glTest[8].iLoad=NUM_SPRITES2;
	glTest[9].bMode=0; glTest[9].bVideo=SDL_SWSURFACE|SDL_FULLSCREEN; glTest[9].iLoad=NUM_SPRITES;
	glTest[10].bMode=2; glTest[10].bVideo=SDL_SWSURFACE|SDL_FULLSCREEN; glTest[10].iLoad=NUM_SPRITES2;
	glTest[11].bMode=2; glTest[11].bVideo=SDL_SWSURFACE|SDL_FULLSCREEN; glTest[11].iLoad=NUM_SPRITES;
	width = 800;
	height = 600;
	video_bpp = 32;
	
	// Cabecera informativa, descripcion, contactos, agredecimientos, informacion del sistema en HTML.
	fprintf(fLog,"<title>%s</title>",GLSDL_VERSION);
	fprintf(fLog,"<table border=1 cellpadding=0 cellspacing=0 style=border-collapse: collapse bordercolor=#0000FF width=600><tr>");
    fprintf(fLog,"<td width=100%><p align=center><b><font color=#000080>%s <br>SDL video benchmark using: directdraw, windib, glSDL, OpenGL and Direct3D</font></b></td>",GLSDL_VERSION);
    fprintf(fLog,"</tr><tr><td width=100%><blockquote><p><font color=#3333FF size=2>- Roberto Prieto - contact@megastormsystems.com <br>");
	fprintf(fLog,"- MegaStorm Systems (c) 2013 - http://www.megastormsystems.com<br>- Using SDL 1.2 experimental (SDL 1.2.9 CVS and glSDL backend) and DirectX<br>");
    fprintf(fLog,"- Using SDL 2.0 with Direct3D, OpenGL and Software renderers<br>");
	fprintf(fLog,"- Special greetings to Sam Latinga, David Olofson and Sthephane Marchesin</font></p></blockquote></td></tr></table>");
    fprintf(fLog,"<font size=2 color=#008080>");
    fprintf(fLog,"<br>");
	GetProcessorInfo();
	GetMemoryInfo();
	GetVideocardInfo();
	fprintf(fLog,"  Screen resolution %dx%dx%dbits<br><br>",width,height,video_bpp);
    fprintf(fLog,"</font><font size=2 color=#808080>");

	// Loop executing all the tests
	iTest = 0; iLast = -1;
	while(iTest < NUM_TEST)
	{	
		// Activamos la variable de entorno correspondiente.
        // DirectX backend
		if((glTest[iTest].bMode == 0) && (glTest[iTest].bMode != iLast))
		{       
			// Cambio de renderer, si no es la primera ejecucion, cerramos el SDL anterior
 		    if(iLast != -1) SDL_Quit();
			// Ponemos variable de entorno
			i = _putenv( "SDL_VIDEODRIVER=directx" );		//	fprintf(fLog,"\nContext change DX %d\n",iTest);
			// Cada cambio de renderer, debemos volver a iniciar
	        if(SDL_Init(SDL_INIT_VIDEO) < 0 ) 
	        {		
		      fprintf(fLog,"  Couldn't initialize SDL: %s<br>",SDL_GetError());		
		      fprintf(fLog,"</font>");
			  fclose(fLog);
			  exit(1);	
	        }
			SDL_ShowCursor(0);
			iLast = glTest[iTest].bMode;
		}
		// glSDL backend, reactiva siempre SDL
		else if(glTest[iTest].bMode == 1)// && (glTest[iTest].bMode!=iLast))
		{
			// Cambio de renderer, si no es la primera ejecucion, cerramos el SDL anterior
 		    if(iLast != -1) SDL_Quit();
			// Ponemos variable de entorno
			i = _putenv("SDL_VIDEODRIVER=glSDL");  //fprintf(fLog,"\nContext change GL %d\n",iTest);
			// Cada cambio de renderer, debemos volver a iniciar
	        if(SDL_Init(SDL_INIT_VIDEO) < 0) 
	        {		
		      fprintf(fLog,"  Couldn't initialize SDL: %s<br>",SDL_GetError());		
		      fprintf(fLog,"</font>");
			  fclose(fLog);
			  exit(1);	
	        }
			SDL_ShowCursor(0);
			iLast = glTest[iTest].bMode;
		}	
		// windib backend
		else if(glTest[iTest].bMode == 2)
		{
			// Cambio de renderer, si no es la primera ejecucion, cerramos el SDL anterior
 		    if(iLast != -1) SDL_Quit();
			// Ponemos variable de entorno
			i = _putenv("SDL_VIDEODRIVER=windib");  //fprintf(fLog,"\nContext change windib %d\n",iTest);
			// Cada cambio de renderer, debemos volver a iniciar
	        if(SDL_Init(SDL_INIT_VIDEO) < 0) 
	        {		
		      fprintf(fLog,"  Couldn't initialize SDL: %s<br>",SDL_GetError());		
		      fprintf(fLog,"</font>");
			  fclose(fLog);
			  exit(1);	
	        }
			SDL_ShowCursor(0);
			iLast = glTest[iTest].bMode;
		}

	    numsprites = glTest[iTest].iLoad;	
		videoflags = glTest[iTest].bVideo;
	
		/* Set video mode */	
		screen = SDL_SetVideoMode(width, height, video_bpp, videoflags);	
		if(!screen ) 
		{		
			fprintf(fLog, "  Couldn't set %dx%d video mode: %s<br>",width, height, SDL_GetError());		
			SDL_Quit();
			fprintf(fLog,"</font>");
			fclose(fLog);
			exit(2);	
		}
	
		/* Load the sprite */	
		if(LoadSprite(screen, "icon.bmp") < 0) { SDL_Quit(); exit(1); }
	
		/* Allocate memory for the sprite info */	
		mem = (Uint8 *)malloc(4*sizeof(SDL_Rect)*numsprites);	
		if ( mem == NULL ) 
		{		
			SDL_FreeSurface(sprite);		
			fprintf(fLog,"  Out of memory!<br>");
			fprintf(fLog,"</font>");
			SDL_Quit();
			fclose(fLog);
			exit(2);	
		}	
		sprite_rects = (SDL_Rect *)mem;	
		positions = sprite_rects;	
		sprite_rects += numsprites;	
		velocities = sprite_rects;	
		sprite_rects += numsprites;	
		sprite_w = sprite->w;	
		sprite_h = sprite->h;	
		srand(time(NULL));	
		for(i = 0; i < numsprites; ++i)
		{		
			positions[i].x = rand()%(screen->w - sprite_w);		
			positions[i].y = rand()%(screen->h - sprite_h);		
			positions[i].w = sprite->w;		
			positions[i].h = sprite->h;		
			velocities[i].x = 0;		
			velocities[i].y = 0;		
			while ( ! velocities[i].x && ! velocities[i].y ) 
			{			
				velocities[i].x = (rand()%(MAX_SPEED*2+1))-MAX_SPEED;			
				velocities[i].y = (rand()%(MAX_SPEED*2+1))-MAX_SPEED;		
			}	
		}	
	
		/* Print out information about our surfaces */	
	    if(glTest[iTest].bMode == 0)
		{
			if((screen->flags & SDL_HWSURFACE) == SDL_HWSURFACE) strcpy(buf,"DirectX/GPU");
			else strcpy(buf,"DirectX/CPU");
		}
		else if(glTest[iTest].bMode == 1) strcpy(buf,"OpenGL");
		else if(glTest[iTest].bMode == 2) strcpy(buf,"Windib/CPU");
		else strcpy(buf,"Unknown");


		#ifdef DEBUG_BACKEND
		 char szRealBE[20]; char *szEnv;
		 SDL_VideoDriverName(szRealBE,20);
		 szEnv = getenv("SDL_VIDEODRIVER");
		 fprintf(fLog,"  Test %d using '%s' engine ('%s'-'%s' backend).<br>",iTest+1,buf,szRealBE,szEnv);
		#else 
   		 fprintf(fLog,"  Test %d using '%s' engine.<br>",iTest+1,buf);
		#endif
		fprintf(fLog,"  -> Number of sprites: %d<br>",glTest[iTest].iLoad);
		fprintf(fLog,"  -> Screen is at %d bits per pixel<br>",screen->format->BitsPerPixel);	
		if ( (screen->flags & SDL_HWSURFACE) == SDL_HWSURFACE )	fprintf(fLog,"  -> Screen is in video memory<br>");	
		else fprintf(fLog,"  -> Screen is in system memory<br>");	
		if ( (screen->flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF )	fprintf(fLog,"  -> Screen has double-buffering enabled<br>");	
		if ( (sprite->flags & SDL_HWSURFACE) == SDL_HWSURFACE ) fprintf(fLog,"  -> Sprite is in video memory<br>");	
		else fprintf(fLog,"  -> Sprite is in system memory<br>");	
	
		/* Run a sample blit to trigger blit acceleration */	
		{ 
			SDL_Rect dst;		
			dst.x = 0;		
			dst.y = 0;		
			dst.w = sprite->w;		
			dst.h = sprite->h;		
			SDL_BlitSurface(sprite, NULL, screen, &dst);		
			SDL_FillRect(screen, &dst, 0x000000);	
		}	
		if ( (sprite->flags & SDL_HWACCEL) == SDL_HWACCEL )	fprintf(fLog,"  -> Sprite blit uses hardware acceleration<br>");	
		if ( (sprite->flags & SDL_RLEACCEL) == SDL_RLEACCEL ) fprintf(fLog,"  -> Sprite blit uses RLE acceleration<br>");
	
		/* Loop, blitting sprites and waiting for a keystroke */	
		frames = 0;	
		then = SDL_GetTicks();	
		done = 0;	
		sprites_visible = 0;	
		while ( !done ) 
		{				
			++frames;
			MoveSprites(screen, 0x00FF00);	
			iTime=SDL_GetTicks();
			if(((iTime-then)/1000)>STEP_TIME) { done=1;  }
		}

		// Mostramos fps, cerramos completamente SDL para evitar leaks y preparamos para otro posible ciclo	
		SDL_FreeSurface(sprite);
		//SDL_FreeSurface(bg);
		free(mem);	
		/* Print out some timing information */	
		now = SDL_GetTicks();	
		glTest[iTest].fps=((double)frames*1000)/(now-then);
		if ( now > then ) fprintf(fLog,"  -> <font color=#FF0000>%2.2f</font> frames per second<br><br>",glTest[iTest].fps);		
		iTest++;
		frames=0;		
	}

	// Cerramos SDL
	SDL_Quit();

    // Mostramos tabla informativa en html
	fprintf(fLog,"<table border=1 cellpadding=0 cellspacing=0 style=border-collapse: collapse bordercolor=#3333FF width=600 height=70><tr>");
	fprintf(fLog,"<td width=196 align=center rowspan=2 height=30>&nbsp;</td><td width=196 align=center colspan=2 height=15><b>");
	fprintf(fLog,"<font size=2 color=#000080>Fullscreen</font></b></td><td width=204 align=center colspan=2 height=15><b>");
    fprintf(fLog,"<font size=2 color=#000080>Fullscreen + doublebuffer</font></b></td></tr><tr>");
    fprintf(fLog,"<td width=98 align=center height=14><font size=2 color=#3333FF>%d</font></td><td width=95 align=center height=14><font size=2 color=#3333FF>%d</font></td>",NUM_SPRITES2,NUM_SPRITES);
    fprintf(fLog,"<td width=104 align=center height=14><font size=2 color=#3333FF>%d</font></td><td width=96 align=center height=14><font size=2 color=#3333FF>%d",NUM_SPRITES2,NUM_SPRITES);
	
	fprintf(fLog,"</font></td></tr><tr><td width=196 align=center height=19><b><font size=2 color=#000080>SDL using DirectX/GPU</font></b></td>");
    fprintf(fLog,"<td width=99 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[2].fps);
	fprintf(fLog,"<td width=94 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[3].fps);
	fprintf(fLog,"<td width=105 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[0].fps);
	fprintf(fLog,"<td width=95 align=center height=19><font size=2 color=#800000>%2.2f</font></td></tr>",glTest[1].fps);
	
	fprintf(fLog,"<tr><td width=196 align=center height=19><b><font size=2 color=#000080>SDL using OpenGL</font></b></td>");
	fprintf(fLog,"<td width=99 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[6].fps);
	fprintf(fLog,"<td width=94 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[7].fps);
	fprintf(fLog,"<td width=105 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[4].fps);
	fprintf(fLog,"<td width=95 align=center height=19><font size=2 color=#800000>%2.2f</font></td></tr>",glTest[5].fps);

	fprintf(fLog,"<tr><td width=196 align=center height=19><b><font size=2 color=#000080>SDL using DirectX/CPU</font></b></td>");
	fprintf(fLog,"<td width=99 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[8].fps);
	fprintf(fLog,"<td width=94 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[9].fps);
	fprintf(fLog,"<td width=105 align=center height=19><font size=2 color=#800000>--</font></td>");
	fprintf(fLog,"<td width=95 align=center height=19><font size=2 color=#800000>--</font></td></tr>");

	fprintf(fLog,"<tr><td width=196 align=center height=19><b><font size=2 color=#000080>SDL using Windib/CPU</font></b></td>");
	fprintf(fLog,"<td width=99 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[10].fps);
	fprintf(fLog,"<td width=94 align=center height=19><font size=2 color=#800000>%2.2f</font></td>",glTest[11].fps);
	fprintf(fLog,"<td width=105 align=center height=19><font size=2 color=#800000>--</font></td>");
	fprintf(fLog,"<td width=95 align=center height=19><font size=2 color=#800000>--</font></td></tr>");
	
    // Execute external SDL 2.0 test
    _putenv("SDL_VIDEODRIVER=");
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );
    if(!CreateProcess( NULL, // No module name (use command line). 
        "sdl20-bench.bin",   // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        NULL,             // Use parent's starting directory. 
        &si,              // Pointer to STARTUPINFO structure.
        &pi ))     		  // Pointer to PROCESS_INFORMATION structure.
    {
        fprintf(fLog,"Warning: could not execute SDL 2.0 benchmark<br>");
    }
    else
    {
        WaitForSingleObject(pi.hProcess,INFINITE);
        CloseHandle(pi.hProcess);
        CloseHandle(pi.hThread);
    }
    
    // Load external results
    FILE *fOut = fopen("sdl20.output", "rt");
    if(fOut != NULL)
    {
        fseek(fOut,0L,SEEK_END);
        int iSize = ftell(fOut);
        fseek(fOut,0L,0);
        char *pBuffer = (char*)malloc(iSize + 1);
        memset(pBuffer,0,iSize);
        fread(pBuffer,sizeof(char),iSize,fOut);
        fclose(fOut);
        fwrite(pBuffer,sizeof(char),iSize,fLog);
        free(pBuffer);
        system("del sdl20.output");
    }
    else fprintf(fLog,"Warning: could not load SDL 2.0 benchmark output<br>");

    // Fin del programa	
	fprintf(fLog,"</table><br>  End of execution<br>");
	fclose(fLog);
	return 0;
}



int LoadSprite(SDL_Surface *screen, char *file)
{
	SDL_Surface *temp;

	/* Load the sprite image */
	sprite = SDL_LoadBMP(file);
	if ( sprite == NULL ) {
		fprintf(stderr, "Couldn't load %s: %s", file, SDL_GetError());
		return(-1);
	}

	/* Set transparent pixel as the pixel at (0,0) */
	if ( sprite->format->palette ) {SDL_SetColorKey(sprite, (SDL_SRCCOLORKEY|SDL_RLEACCEL),*(Uint8 *)sprite->pixels);}
	else SDL_SetColorKey(sprite, (SDL_SRCCOLORKEY|SDL_RLEACCEL),0);

	/* Convert sprite to video format */
	temp = SDL_DisplayFormat(sprite);
	SDL_FreeSurface(sprite);
	if ( temp == NULL ) 
	{
		fprintf(stderr, "Couldn't convert background: %s\n",
							SDL_GetError());
		return(-1);
	}
	sprite = temp;

	// Creamos fondo para borrar todo(el fillrect es lento)
	//p=SDL_CreateRGBSurface(screen->flags, screen->w, screen->h, screen->format->BitsPerPixel,screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);
	//bg=SDL_DisplayFormat(temp);
	//SDL_FillRect(bg,NULL,0);

	/* We're ready to roll. :) */
	return(0);
}

void MoveSprites(SDL_Surface *screen, Uint32 background)
{
	int i, nupdates;
	SDL_Rect area, *position, *velocity;

	nupdates = 0;
	
	SDL_FillRect(screen, NULL, 0xA0A0A0FF);
	//SDL_BlitSurface(bg,NULL,screen,NULL);
	

	/* Move the sprite, bounce at the wall, and draw */
	for ( i=0; i<numsprites; ++i ) {
		position = &positions[i];
		velocity = &velocities[i];
		position->x += velocity->x;
		if ( (position->x < 0) || (position->x >= (screen->w - sprite_w)) ) {
			velocity->x = -velocity->x;
			position->x += velocity->x;
		}
		position->y += velocity->y;
		if ( (position->y < 0) || (position->y >= (screen->h - sprite_w)) ) {
			velocity->y = -velocity->y;
			position->y += velocity->y;
		}

		/* Blit the sprite onto the screen */
		area = *position;
		SDL_BlitSurface(sprite, NULL, screen, &area);
		sprite_rects[nupdates++] = area;
	}

#if DEBUG_FLIP
    {
	if ( (screen->flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF ) {
            static int t = 0;

            Uint32 color = SDL_MapRGB (screen->format, 0, 0, 255);
            SDL_Rect r;
            r.x = (sin((float)t * 2 * 3.1459) + 1.0) / 2.0 * (screen->w-20);
            r.y = 0;
            r.w = 20;
            r.h = screen->h;
        
            SDL_FillRect (screen, &r, color);
            t+=2;
        }
    }
#endif
    
	// Update the screen!
	/*if ( (screen->flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF ) {
		SDL_Flip(screen);
	} else {
		SDL_UpdateRects(screen, nupdates, sprite_rects);
	}
	sprites_visible = 1;
    */
    SDL_Flip(screen);
}


// Support functions to get system information

bool GetProcessorInfo()
{
	LONG result;
	HKEY hKey;
	TCHAR vendorData [128];
	DWORD data;
	DWORD dataSize;
	char sCPU[128];
	char sSpeed[64];
	char sTmp[128];

	// Get the processor speed info.
	result = ::RegOpenKeyEx (HKEY_LOCAL_MACHINE,"Hardware\\Description\\System\\CentralProcessor\\0", 0, KEY_QUERY_VALUE, &hKey);

	// Check if the function has succeeded.
	if (result == ERROR_SUCCESS) 
	{
		dataSize=sizeof(data);
		result = ::RegQueryValueEx(hKey, "~MHz", NULL, NULL,(LPBYTE)&data, &dataSize);
		sprintf(sTmp,"%d",data);
		strcpy(sSpeed,"  CPU Speed: ");
		strcat(sSpeed,sTmp);
		strcat(sSpeed," Mhz");
		
		dataSize=sizeof (vendorData);
		result = ::RegQueryValueEx (hKey, "ProcessorNameString", NULL, NULL,(LPBYTE)vendorData, &dataSize);
		strcpy(sCPU,"  ");
		sprintf(sTmp,"%s",vendorData);
		// To cut left blank spaces
		result=0;
		while(sTmp[result]==' ') result++;
		memcpy(sTmp,sTmp+result,strlen(sTmp)-result);

		strcat(sCPU,sTmp);
		strcat(sCPU," (");

		dataSize=sizeof (vendorData);
		result = ::RegQueryValueEx (hKey, "Identifier", NULL, NULL,(LPBYTE)vendorData, &dataSize);
		sprintf(sTmp,"%s",vendorData);
		strcat(sCPU,sTmp);
		strcat(sCPU,")");		
	}

	// Make sure to close the reg key
	RegCloseKey (hKey);
	// Print info
	fprintf(fLog,"%s<br>",sCPU);
    fprintf(fLog,"%s<br>",sSpeed);
	return (true);
}


bool GetMemoryInfo ()
{
	Uint32 var;
	char sTmp[32];
	char sMem[64];
	MEMORYSTATUS memoryStatus;
	memset (&memoryStatus, sizeof (MEMORYSTATUS), 0);
	memoryStatus.dwLength = sizeof (MEMORYSTATUS);
	GlobalMemoryStatus (&memoryStatus);
	strcpy(sMem,"  System memory total/available: ");
	var = memoryStatus.dwTotalPhys/1024/1024;
	sprintf(sTmp,"%d",var);
	strcat(sMem,sTmp);
	strcat(sMem,"Mb / ");

	var = memoryStatus.dwAvailPhys/1024/1024;
	sprintf(sTmp,"%d",var);
	strcat(sMem,sTmp);
	strcat(sMem,"Mb");

	// Print info
	fprintf(fLog,"%s<br>",sMem);
	return (true);
}


bool GetVideocardInfo()
{
	char sVideo[256];

	// Initialize SDL
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) return false;

	if ( SDL_SetVideoMode( 640, 480, 32,SDL_OPENGL|SDL_NOFRAME) == NULL ) 
	{
		strcpy(sVideo,"  Couldn't detect videocard (");
		strcat(sVideo, SDL_GetError());
		strcat(sVideo,")");
	}
	else
	{
		strcpy(sVideo,"  ");
		strcat(sVideo,(const char*)glGetString(GL_VENDOR));
		strcat(sVideo," - ");
		strcat(sVideo,(const char*)glGetString(GL_RENDERER));
		strcat(sVideo," - ");
		strcat(sVideo,(const char*)glGetString(GL_VERSION));		
	}

	// print info
	SDL_Quit();
	fprintf(fLog,"%s<br>",sVideo);
	return (true);
}