/* Simple program:  Move N sprites around on the screen as fast as possible */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <time.h>
#include <windows.h>
#include "SDL.h"

#define DEBUG_FLIP 1

#define NUM_SPRITES	1000
#define NUM_SPRITES2 100
#define MAX_SPEED 	1

SDL_Surface *sprite;
int numsprites;
SDL_Rect *sprite_rects;
SDL_Rect *positions;
SDL_Rect *velocities;
int sprites_visible;
Uint16 sprite_w, sprite_h;
const SDL_VideoInfo *vi;
char buf[20];

#define NUM_TEST 4
struct glTest
{
	Uint16 bMode;  // 0: DirectX 1: glSDL
	Uint32 bVideo; // SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF
	Uint16 iLoad;  // 1000 sprites or 100 sprites
} glTest[NUM_TEST];

int LoadSprite(SDL_Surface *screen, char *file)
{
	SDL_Surface *temp;

	/* Load the sprite image */
	sprite = SDL_LoadBMP(file);
	if ( sprite == NULL ) {
		fprintf(stderr, "Couldn't load %s: %s", file, SDL_GetError());
		return(-1);
	}

	/* Set transparent pixel as the pixel at (0,0) */
	if ( sprite->format->palette ) {
		SDL_SetColorKey(sprite, (SDL_SRCCOLORKEY|SDL_RLEACCEL),
						*(Uint8 *)sprite->pixels);
	}
	else SDL_SetColorKey(sprite, (SDL_SRCCOLORKEY|SDL_RLEACCEL),0);

	/* Convert sprite to video format */
	temp = SDL_DisplayFormat(sprite);
	SDL_FreeSurface(sprite);
	if ( temp == NULL ) {
		fprintf(stderr, "Couldn't convert background: %s\n",
							SDL_GetError());
		return(-1);
	}
	sprite = temp;

	/* We're ready to roll. :) */
	return(0);
}

void MoveSprites(SDL_Surface *screen, Uint32 background)
{
	int i, nupdates;
	SDL_Rect area, *position, *velocity;

	nupdates = 0;
	/* Erase all the sprites if necessary */
	if ( sprites_visible ) {
		SDL_FillRect(screen, NULL, background);
	}

	/* Move the sprite, bounce at the wall, and draw */
	for ( i=0; i<numsprites; ++i ) {
		position = &positions[i];
		velocity = &velocities[i];
		position->x += velocity->x;
		if ( (position->x < 0) || (position->x >= (screen->w - sprite_w)) ) {
			velocity->x = -velocity->x;
			position->x += velocity->x;
		}
		position->y += velocity->y;
		if ( (position->y < 0) || (position->y >= (screen->h - sprite_w)) ) {
			velocity->y = -velocity->y;
			position->y += velocity->y;
		}

		/* Blit the sprite onto the screen */
		area = *position;
		SDL_BlitSurface(sprite, NULL, screen, &area);
		sprite_rects[nupdates++] = area;
	}

#if DEBUG_FLIP
    {
	if ( (screen->flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF ) {
            static int t = 0;

            Uint32 color = SDL_MapRGB (screen->format, 255, 0, 0);
            SDL_Rect r;
            r.x = (sin((float)t * 2 * 3.1459) + 1.0) / 2.0 * (screen->w-20);
            r.y = 0;
            r.w = 20;
            r.h = screen->h;
        
            SDL_FillRect (screen, &r, color);
            t+=2;
        }
    }
#endif
    
	/* Update the screen! */
	if ( (screen->flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF ) {
		SDL_Flip(screen);
	} else {
		SDL_UpdateRects(screen, nupdates, sprite_rects);
	}
	sprites_visible = 1;
}

int main(int argc, char *argv[])
{
	SDL_Surface *screen;
	Uint8 *mem;
	int width, height;
	Uint8  video_bpp;
	Uint32 videoflags;
	Uint32 background;
	int    i, done,iTest;
	SDL_Event event;
	Uint32 then, now, frames;

	// Inicializa sistema de test	
	glTest[0].bMode=0; glTest[0].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF; glTest[0].iLoad=100;
	glTest[1].bMode=1; glTest[1].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF; glTest[1].iLoad=1000;
	glTest[2].bMode=0; glTest[2].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN; glTest[2].iLoad=100;
	glTest[3].bMode=0; glTest[3].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN; glTest[3].iLoad=1000;
	glTest[4].bMode=1; glTest[4].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF; glTest[4].iLoad=100;
	glTest[5].bMode=1; glTest[5].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF; glTest[5].iLoad=1000;
	glTest[6].bMode=1; glTest[6].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN; glTest[6].iLoad=100;
	glTest[7].bMode=1; glTest[7].bVideo=SDL_HWSURFACE|SDL_FULLSCREEN; glTest[7].iLoad=1000;
	width = 800;
	height = 600;
	video_bpp = 32;
	printf(" -------------------------------------------------------------\n");
	printf("      glTest - Modified SDL example to test glSDL backend     \n");
	printf("   Coded by Roberto Prieto - http://www.megastormsystems.com  \n");
	printf(" -------------------------------------------------------------\n\n");

	// Loop with 8 test
	iTest=0;
	while(iTest<2) //NUM_TEST)
	{	
/*
Calling SetEnvironmentVariable has no effect on the system environment variables. The user can add or modify system environment variables using the Control Panel. To programmatically add or modify system environment variables, add them to the HKEY_LOCAL_MACHINE\System\CurrentControlSet\Control\Session Manager\Environment registry key, then broadcast a WM_SETTINGCHANGE message. This allows applications, such as the shell, to pick up your updates.
*/

		// Activamos la variable de entorno correspondiente
        if(glTest[iTest].bMode==0) i=SetEnvironmentVariable("SDL_VIDEODRIVER","directx");
		else i=SetEnvironmentVariable("SDL_VIDEODRIVER","glSDL");
		printf("i: %d\n",i);

		/* Initialize SDL */	
		if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
		{		
			fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());		
			exit(1);	
		}

        numsprites = glTest[iTest].iLoad;	
		videoflags = glTest[iTest].bVideo;
	
		/* Set video mode */	
		screen = SDL_SetVideoMode(width, height, video_bpp, videoflags);	
		if ( ! screen ) 
		{		
			fprintf(stderr, "Couldn't set %dx%d video mode: %s\n",width, height, SDL_GetError());		
			SDL_Quit();
			exit(2);	
		}
	
		/* Load the sprite */	
		if ( LoadSprite(screen, "icon.bmp") < 0 ) { SDL_Quit(); exit(1); }
	
		/* Allocate memory for the sprite info */	
		mem = (Uint8 *)malloc(4*sizeof(SDL_Rect)*numsprites);	
		if ( mem == NULL ) 
		{		
			SDL_FreeSurface(sprite);		
			fprintf(stderr, "Out of memory!\n");
			SDL_Quit();
			exit(2);	
		}	
		sprite_rects = (SDL_Rect *)mem;	
		positions = sprite_rects;	
		sprite_rects += numsprites;	
		velocities = sprite_rects;	
		sprite_rects += numsprites;	
		sprite_w = sprite->w;	
		sprite_h = sprite->h;	
		srand(time(NULL));	
		for ( i=0; i<numsprites; ++i )
		{		
			positions[i].x = rand()%(screen->w - sprite_w);		
			positions[i].y = rand()%(screen->h - sprite_h);		
			positions[i].w = sprite->w;		
			positions[i].h = sprite->h;		
			velocities[i].x = 0;		
			velocities[i].y = 0;		
			while ( ! velocities[i].x && ! velocities[i].y ) 
			{			
				velocities[i].x = (rand()%(MAX_SPEED*2+1))-MAX_SPEED;			
				velocities[i].y = (rand()%(MAX_SPEED*2+1))-MAX_SPEED;		
			}	
		}	
		background = SDL_MapRGB(screen->format, 0xF0, 0x00, 0x00);
	
		/* Print out information about our surfaces */	
	    vi=SDL_GetVideoInfo();
	    SDL_VideoDriverName(buf,20);
		printf("  GFX engine: %s\n",buf);
		printf("  -> Number of sprites: %d\n",glTest[iTest].iLoad);
		printf("  -> Screen is at %d bits per pixel\n",screen->format->BitsPerPixel);	
		if ( (screen->flags & SDL_HWSURFACE) == SDL_HWSURFACE )	printf("  -> Screen is in video memory\n");	
		else printf("  -> Screen is in system memory\n");	
		if ( (screen->flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF )	printf("  -> Screen has double-buffering enabled\n");	
		if ( (sprite->flags & SDL_HWSURFACE) == SDL_HWSURFACE ) printf("  -> Sprite is in video memory\n");	
		else printf("  -> Sprite is in system memory\n");	
	
		/* Run a sample blit to trigger blit acceleration */	
		{ 
			SDL_Rect dst;		
			dst.x = 0;		
			dst.y = 0;		
			dst.w = sprite->w;		
			dst.h = sprite->h;		
			SDL_BlitSurface(sprite, NULL, screen, &dst);		
			SDL_FillRect(screen, &dst, background);	
		}	
		if ( (sprite->flags & SDL_HWACCEL) == SDL_HWACCEL )	printf("  -> Sprite blit uses hardware acceleration\n");	
		if ( (sprite->flags & SDL_RLEACCEL) == SDL_RLEACCEL ) printf("  -> Sprite blit uses RLE acceleration\n");
	
		/* Loop, blitting sprites and waiting for a keystroke */	
		frames = 0;	
		then = SDL_GetTicks();	
		done = 0;	
		sprites_visible = 0;	
		while ( !done ) 
		{		
			/* Check for events */		
			++frames;		
			while ( SDL_PollEvent(&event) ) 
			{			
				switch (event.type) 
				{				
				case SDL_MOUSEBUTTONDOWN:
				case SDL_QUIT:
					done = 1;
					break;
				default:
					break;			
				}		
			}		
			MoveSprites(screen, background);	
		}

		// Mostramos fps, cerramos completamente SDL para evitar leaks y preparamos para otro posible ciclo	
		SDL_FreeSurface(sprite);	
		free(mem);	
		/* Print out some timing information */	
		now = SDL_GetTicks();	
		if ( now > then ) printf("  -> %2.2f frames per second\n\n",((double)frames*1000)/(now-then));
		iTest++;
		frames=0;
		SDL_Quit();
	}
	
	// Fin del programa
	printf("  End of execution\n");
	return(0);
}
